package pfc;
import java.util.Vector;

import javax.xml.bind.annotation.XmlElement;

/** pfc.DistribucioHabitacio
 * 
 * Representa la distribucio de persones a una habitacio.
 * Tenim l'edat de cada ocupant a l'habitacio.
 * Existeix una edat per defecte en cas de que no s'indiqui edat per a un ocupant.
 */
public class DistribucioHabitacio {
	private Vector<Integer> edats = null;

	public DistribucioHabitacio(){
		this.edats = new Vector<Integer>();
	}

	/** Afegeix un ocupant a l'habitacio
	 * @param edat Edat de l'ocupant
	 */
	public void add(int edat){
		this.edats.add(edat);
	}

	/**
	 * @return Edats dels ocupants de l'habitacio
	 */
	public Vector<Integer> getEdats() {
		return this.edats;
	}

	/**
	 * @return Nombre d'ocupants a l'habitacio
	 */
	public int numPax() {
		return this.edats.size();
	}

	/**
	 * @return Nombre d'ocupants adults a l'habitacio
	 */
	public int getNumAdults(){
		return getNumAdults(pfc.conf.Constants.DEFAULT_EDAT_ADULT);
	}

	/**
	 * @param edatAdult Edat minima d'un adult
	 * @return Nombre d'ocupants adults a l'habitacio
	 */
	public int getNumAdults(int edatAdult) {
		int result = 0;
		for (Integer pos: edats){
			if (pos >= edatAdult) {
				result ++;
			}
		}
		return result;
	}

	/**
	 * @return Nombre d'ocupants nins a l'habitacio
	 */
	public int getNumNins(){
		return this.edats.size() - this.getNumAdults();
	}

	/**
	 * @param edatAdult Edat minima d'un adult
	 * @return Nombre d'ocupants nins a l'habitacio
	 */
	public int getNumNins(int edatAdult){
		return this.edats.size() - this.getNumAdults(edatAdult);
	}

	/**
	 * @param nin Posicio del nin
	 * @return Edat del nin ocupant indicat
	 */
	public int getEdatNin(int nin){
		return this.getEdatNin(nin, pfc.conf.Constants.DEFAULT_EDAT_ADULT);
	}

	/**
	 * @param nin Posicio del nin
	 * @param edatAdult Edat minima d'un adult
	 * @return Edat del nin ocupant indicat
	 */
	public int getEdatNin(int nin, int edatAdult){
		int found = -1;
		for (Integer pos: edats){
			if (pos < edatAdult) {
				found++;
				if (found == nin) return pos;
			}
		}
		return -1;
	}

	/**
	 * @param edatInicial Edat minima
	 * @param edatFinal Edat maxima
	 * @return Nombre d'ocupants entre les edats indicades
	 */
	public int getNumTotalPax(int edatInicial, int edatFinal) {
		int found = 0;
		for(Integer pos: edats){
			if (pos >= edatInicial && pos <= edatFinal) {
				found++;
			}
		}
		return found;
		
	}

	/**
	 * @param edat Edat maxima de l'infant
	 * @return Nombre total d'infants
	 */
	public int getNumBebes(int edat) {
		int found = 0;
		for(Integer pos: edats){
			if (pos <= edat) {
				found++;
			}
		}
		return found;
		
	}

	public String toString(){
		return getRepresentacion();
	}

	@XmlElement(name="representacio")
	public String getRepresentacion(){
		StringBuffer sb = new StringBuffer();
		sb.append("[");
		if (this.edats!=null) {
			for (int i=0; i<this.edats.size(); i++) {
				if (i>0) sb.append(",");
				sb.append(this.edats.get(i));
			}
		}
		sb.append("]");
		return sb.toString();
	}
}