package pfc;

import java.io.IOException;
import java.io.InputStream;
import java.util.Calendar;
import java.util.Properties;
import java.util.Vector;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.PropertyConfigurator;

import pfc.exception.PeticioConsultaException;
import pfc.exception.SenseResultatsException;

@SuppressWarnings("serial")
/** pfc.Pfc
 * 
 * Servlet principal de l'aplicacio (entrada al sistema). 
 * Implementa els metodes d'inicialitzacio i destruccio i les respostes davant metodes GET i POST.
 * 
 */
public class Pfc extends HttpServlet {

	private ServletContext servletContext;

	public Pfc() {
		super();
	}

	public void destroy() {
		System.out.println("Servlet destroy");
		Cercador.shutdown();
	}

	public void init(ServletConfig config) throws ServletException {

		System.out.println("Servlet init");
		servletContext = config.getServletContext();

		//Inicialitzacio de la propietat de sistema "JBOSS_HOME" (path del servidor JBoss)
		String JBOSS_HOME = System.getenv("JBOSS_HOME");
		System.out.println("Servlet init...[JBOSS_HOME="+JBOSS_HOME + "]");
		if (JBOSS_HOME == null) JBOSS_HOME = "C:\\jboss-5.1.0.GA\\";
		System.setProperty("JBOSS_HOME", JBOSS_HOME);

		try {
			//Inicialitzacio de LOG4J
			//Carrega del fitxer de configuracio
			InputStream inputStreamLOG4J = Thread.currentThread().getContextClassLoader().getResourceAsStream(pfc.conf.Constants.PATH_PROPERTIES_LOG4J);
			Properties perfilBundleLOG4J = new Properties();
			perfilBundleLOG4J.load(inputStreamLOG4J);
			PropertyConfigurator.configure(perfilBundleLOG4J);
	    }
		catch (IOException iOe)
		{
			System.out.println("Fatal error. Log4j maybe not initialized.");
			iOe.printStackTrace();
		}

		//Creacio de l'instancia del cercador
		try {
			Cercador.getInstance();
			System.out.println("Servlet init cercador instance successfully created");
		} catch (Exception e) {
			Cercador.shutdown();
			System.out.println("Servlet init cercador instance not created");				
		}

		System.out.println("Servlet initialized");
	}

	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException,IOException {
		System.out.println("PfcController.doGet() CALL");
		RequestDispatcher rd = servletContext.getRequestDispatcher(response.encodeURL("/index.jsp"));
		rd.forward(request, response);
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response)	throws ServletException,IOException {

		System.out.println("PfcController.doPost() CALL -- [" + request.getRequestURI() + ":" + request.getLocalAddr() + "]");

		String error = null;
		String html  = null;

		try {
			//La codificacio del request es UTF-8, la mateixa que la de la JSP index.jsp
			//(Les ciutats poden contenir accents i/o altres caracters especials)
			request.setCharacterEncoding("UTF-8");

			/*
			 * LECTURA DELS PARAMETRES DEL REQUEST
			 * (Es carreguen de nou al request per a seleccionar de nou els valors)
			 */
			String origen = request.getParameter("origen");
			request.setAttribute("origen", origen);

			String desti  = request.getParameter("desti");
			request.setAttribute("desti",  desti);

			String habitacions  = request.getParameter("habitacions");
			request.setAttribute("habitacions",  habitacions);

			String dia_inici = request.getParameter("dia_inici");
			request.setAttribute("dia_inici", dia_inici);

			String mes_inici = request.getParameter("mes_inici");
			request.setAttribute("mes_inici", mes_inici);

			String dia_fi = request.getParameter("dia_fi");
			request.setAttribute("dia_fi", dia_fi);

			String mes_fi = request.getParameter("mes_fi");
			request.setAttribute("mes_fi", mes_fi);

			//Vector amb la distribucio de cada habitacio
			Vector<DistribucioHabitacio> vDistribucionsHabitacions = new Vector<DistribucioHabitacio>();
			//Vector amb una descripcio de cada habitacio segons els seus ocupants
			Vector<String> vTxtDistribucioHabitacions = new Vector<String>();

			//LECTURA DE LES HABITACIONS, OCUPANTS I EDATS
			int iNumHabitacions = Integer.parseInt(habitacions);
			if (iNumHabitacions>pfc.conf.Constants.MAX_ROOMS) 
				//Es comproba que el nombre d'habitacions no superi un cert maxim
				//S'eviten tambe bucles infinits en cas que el nombre fos massa alt (un usuari pot modificar el request)
				throw new Exception(pfc.conf.Constants.MSG_ERROR_REQUEST_LIMITHABITACIONS);
			else {
				
				for (int i=0; i<iNumHabitacions; i++){
					String sNumPaxHabitacio = request.getParameter("paxHab_"+i);
					request.setAttribute("paxHab_"+i, sNumPaxHabitacio);
					int iNumPaxHabitacio = Integer.parseInt(sNumPaxHabitacio);

					Vector<String> edatsNins = new Vector<String>();
					DistribucioHabitacio distribucioHabitacio = new DistribucioHabitacio();
					for (int j=0; j<iNumPaxHabitacio; j++) {
						String sEdat = request.getParameter("paxHab_"+i+"_"+j);
						request.setAttribute("paxHab_"+i+"_"+j, sEdat);
						if (sEdat.equals("")) {
							//Adult (l'edat d'un ocupant es opcional)
							distribucioHabitacio.add(pfc.conf.Constants.DEFAULT_EDAT_PAX);
						}
						else {
							edatsNins.add(sEdat);
							distribucioHabitacio.add(Integer.parseInt(sEdat));
						}
					}
					vDistribucionsHabitacions.add(distribucioHabitacio);

					//Es forma el text de descripcio de habitacions
					String descTotal = iNumPaxHabitacio + (iNumPaxHabitacio>1?" persones" : " persona");
					String descNins  = "";
					for (int indexEdat = 0; indexEdat<edatsNins.size(); indexEdat++) {
						descNins += (indexEdat>0?", ":"")+edatsNins.get(indexEdat);
					}
					if (!descNins.equals("")) {
						descNins = " (" + edatsNins.size() + (edatsNins.size()==1?" nin d'edat ":" nins d'edats ") + descNins + ")";
					}
					vTxtDistribucioHabitacions.add("1 habitaci de " + descTotal + descNins);

				}
			}

			int diaInici = Integer.parseInt(dia_inici);
			int mesInici = Integer.parseInt(mes_inici.split("-")[0]);
			int anyInici = Integer.parseInt(mes_inici.split("-")[1]);
			int diaFi    = Integer.parseInt(dia_fi);
			int mesFi    = Integer.parseInt(mes_fi.split("-")[0]);
			int anyFi    = Integer.parseInt(mes_fi.split("-")[1]);

			//Construccio de la peticio
			BeanPeticioConsulta beanPeticioConsulta = new BeanPeticioConsulta
			(
				diaInici, mesInici, anyInici,
				diaFi, mesFi, anyFi,
				origen, desti,
				vDistribucionsHabitacions, vTxtDistribucioHabitacions
			);

			//Validacio dels parametres
			beanPeticioConsulta.assertPeticio(diaInici,mesInici,anyInici,diaFi,mesFi,anyFi);

			//Execucio del motor de cerca
			long tInici = Calendar.getInstance().getTimeInMillis();
			html = Cercador.getInstance().search(beanPeticioConsulta);
			long tFi = Calendar.getInstance().getTimeInMillis();
			System.out.println("["+beanPeticioConsulta.getId()+"] execution time: '"+(tFi - tInici)+" ms'");
		}
		catch (PeticioConsultaException pce){
			error = pce.getMessage();
			System.out.println("pfc.Pfc.doPost() Error en la peticio: '" + pce.getMessage() + "'");
		}
		catch (SenseResultatsException sre){
			error = sre.getMessage();
			System.out.println("pfc.Pfc.doPost() Sense resultats: '" + sre.getMessage() + "'");
		}
		catch (Exception e) {
			error = "Error inesperat";
			System.out.println("pfc.Pfc.doPost() Fatal error: '" + e.getMessage() + "'");
			e.printStackTrace();
		}
		if (error != null) {
			//Redireccio a la hom amb el missatge d'error
			request.setAttribute("info_error", error);
			RequestDispatcher rd = servletContext.getRequestDispatcher(response.encodeURL("/index.jsp")); //java.net.URLEncoder.encode("/cerca.jsp","UTF-8")
			rd.forward(request, response);
		} else {
			response.setCharacterEncoding("UTF-8");
			response.getWriter().println(html);
		}
	}
}