package pfc.bean;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlTransient;

import pfc.BeanPeticioConsulta;

/** pfc.bean.ItemBean
 * 
 * Un item consultat (un resultat que pot ser una estancia en hotel o un vol)
 * 
 */
@XmlTransient
public abstract class ItemBean implements Comparable<ItemBean> {

	protected BeanPeticioConsulta beanPeticioConsulta;

	//Metode que ha d'implementar cada item de cada operador (com es calcula el preu total)
	@XmlElement(name="preuOriginal")
	public abstract float getPreu();

	//Metode que ha d'implementar cada item de cada operador (el nom de l'operador)
	@XmlElement(name="operador")
	public abstract String getNomOperador();

	//Metodes que que obtenen una representacio String/Html de l'element (a tasques de debug)
	//Metodes implementats a les classes abstractes pfc.bean.HotelBean i pfc.bean.FlightBean
	public abstract String toDebugHTML();

	//Metodes implementats a les classes concretes filles de pfc.bean.flight i pfc.bean.hotel
	protected abstract String getDebugHTML();

	//Retorna l'html que descriu l'item i que es visualitza a la resposta
	@XmlElement(name="html")
	public abstract String getHTML();

	//Key (identificador) de l'item. Els hotels venen representats pel seu nom.
	public abstract String getKey();

	public ItemBean(){
	}

	public ItemBean(BeanPeticioConsulta beanPeticioConsulta){
		this.beanPeticioConsulta = beanPeticioConsulta;
	}	

	@XmlElement(name="preu")
	public String getFormatPreu() {
		return pfc.conf.Util.formatPreu(this.getPreu());
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(ItemBean itemBean) {
		if (this.getPreu() > itemBean.getPreu()) return 1;
		if (this.getPreu() < itemBean.getPreu()) return -1;
		return 0;
	}

}