package pfc.bean.flight;

import java.util.Vector;
import pfc.BeanPeticioConsulta;

/** pfc.bean.flight.EdreamsBean
 * 
 * Item resultat de l'operador eDreams
 */
public class EdreamsBean extends FlightBean {

	//Preu (obligatori)
	private String preu = null;
	//Data d'anada (opcional)
	private String dataAnada = null;
	//anades (obligatori, pero si la expressio regular canvia pot no retornar resultats)
	//Cada anada = [ciutat sortida, hora sortida, ciutat arrivada, hora arrivada, companyia, nombre escales]
	private Vector<String[]> anades = null;
	//Data de tornada (opcional)
	private String dataTornada = null;
	//tornades (obligatori, pero si la expressio regular canvia pot no retornar resultats)
	//Cada tornada = [ciutat sortida, hora sortida, ciutat arrivada, hora arrivada, companyia, nombre escales]
	private Vector<String[]> tornades = null;

	public EdreamsBean(BeanPeticioConsulta beanPeticioConsulta) {
		super(beanPeticioConsulta);
	}

	public float getPreu() {
		return Float.valueOf(preu.replace(",",".")) * beanPeticioConsulta.getNumTotalPax();
	}

	protected String getDebugHTML() {
		StringBuffer sb = new StringBuffer();
		sb.append("<span>Preu: '" + this.preu + "'</span><br />");
		sb.append("<span>Data anada: '" + this.dataAnada + "'</span><br />");
		sb.append("<span>Anades:");
		for (String[] anada: anades) {
			sb.append("<br />[");
			for (int j=0; j<anada.length; j++) sb.append((j>0?",":"")+"("+anada[j]+")");
			sb.append("]");
		}
		sb.append("</span><br />");
		sb.append("<span>Data tornada: '" + this.dataTornada + "'</span><br />");
		sb.append("<span>Tornades:");
		for (String[] tornada: tornades) {
			sb.append("<br />[");
			for (int j=0; j<tornada.length; j++) sb.append((j>0?",":"")+"("+tornada[j]+")");
			sb.append("]");
		}
		sb.append("</span>");
		return sb.toString();
	}

	public String getHTML() {
		StringBuffer sb = new StringBuffer();
		//Anades
		if (this.dataAnada!=null) {
			sb.append("<div><span class=\"s0\">Data anada</span><span>" + this.dataAnada + "</span></div>");
		}
		for (int i=1; i<=this.anades.size(); i++) {
			String[] anada = this.anades.get(i-1);
			sb.append("<div><span class=\"s0\">Anada " + i + "</span></div>");
			sb.append("<div><span class=\"s0\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Horari</span><span>");
			sb.append(anada[0]+" "+anada[1]+" - "+anada[2]+" "+anada[3]);
			sb.append("</span></div>");
			sb.append("<div><span class=\"s0\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Companyia</span><span>");
			sb.append(anada[4]);
			if (anada[5]!=null) sb.append(" (" + anada[5] + " escales)");
			sb.append("</span></div>");
		}
		//Tornades
		if (this.dataTornada!=null) {
			sb.append("<div><span class=\"s0\">Data tornada</span><span>" + this.dataTornada + "</span></div>");
		}
		for (int i=1; i<=this.tornades.size(); i++) {
			String[] tornada = this.tornades.get(i-1);
			sb.append("<div><span class=\"s0\">Tornada " + i + "</span></div>");
			sb.append("<div><span class=\"s0\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Horari</span>");
			sb.append(tornada[0]+" "+tornada[1]+" - "+tornada[2]+" "+tornada[3]);
			sb.append("</span></div>");
			sb.append("<div><span class=\"s0\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Companyia</span><span>");
			sb.append(tornada[4]);
			if (tornada[5]!=null) sb.append(" (" + tornada[5] + " escales)");
			sb.append("</span></div>");
		}
		return sb.toString();
	}

	public String getNomOperador() {
		return "eDreams";
	}
	public void setPreu(String preu) {
		this.preu = preu;
	}
	public void setDataAnada(String dataAnada) {
		this.dataAnada = dataAnada;
	}
	public void setAnades(Vector<String[]> anades) {
		this.anades = anades;
	}
	public void setDataTornada(String dataTornada) {
		this.dataTornada = dataTornada;
	}
	public void setTornades(Vector<String[]> tornades) {
		this.tornades = tornades;
	}
}