package pfc.http;

import java.util.List;

import org.apache.http.HttpRequest;
import org.apache.http.client.CookieStore;
import org.apache.http.client.cache.CacheResponseStatus;
import org.apache.http.cookie.Cookie;
import org.apache.http.impl.client.cache.CachingHttpClient;
import org.apache.http.protocol.HttpContext;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

public class HttpLog {

	private final static Logger loger = Logger.getLogger("http");

	/* 
	 * PARAMETRITZACIO DEBUG 
	 */
	//Flag d'activacio del debug de les respostes HTTP a fitxers HTML
	private static boolean DEBUG_HTMLHTTP_ACTIVAT	= true;
	//Flag d'activacio del debug del tractament amb expressions regulars l l'ordenacio d'elements
	private static boolean DEBUG_HTMLITEMS_ACTIVAT 	= false;
	//Flag d'activacio del debug de les cookies tramesses
	private static boolean DEBUG_COOKIES_ACTIVAT 	= false;
	//Flag d'activacio del debug de les crides HTTP (si esta activat es llegueix l'informacio d'uns fitxers i no es fan crides HTTP)
	private static boolean DEBUG_SCRAPING_ACTIVAT 	= false;

	public static void setDebugHtmlHttpActivat(boolean debugHtmlActivat) {
		pfc.http.HttpLog.DEBUG_HTMLHTTP_ACTIVAT = debugHtmlActivat;
	}
	public static boolean isDebugHtmlHTTPActivat() {
		return DEBUG_HTMLHTTP_ACTIVAT;
	}
	public static void setDebugHtmlItemsActivat(boolean debugHtmlItemsActivat) {
		pfc.http.HttpLog.DEBUG_HTMLITEMS_ACTIVAT = debugHtmlItemsActivat;
	}
	public static boolean isDebugHtmlItemsActivat() {
		return DEBUG_HTMLITEMS_ACTIVAT;
	}
	public static void setDebugCookiesActivat(boolean debugCookiesActivat) {
		pfc.http.HttpLog.DEBUG_COOKIES_ACTIVAT = debugCookiesActivat;
	}
	public static boolean isDebugCookiesActivat() {
		return DEBUG_COOKIES_ACTIVAT;
	}
	public static void setDebugScrapingActivat(boolean debugScrapingActivat) {
		pfc.http.HttpLog.DEBUG_SCRAPING_ACTIVAT = debugScrapingActivat;
	}
	public static boolean isDebugScrapingActivat() {
		return DEBUG_SCRAPING_ACTIVAT;
	}

	/** Inclou el listat de cookies al fitxer de log
	 * @param cookieStore Magatzem amb les cookies
	 */
	public static void printCookies(CookieStore cookieStore){
		if (DEBUG_COOKIES_ACTIVAT){
			List<Cookie> cookies = cookieStore.getCookies();
	        if (cookies.isEmpty()) {
	        	loger.log(Level.DEBUG, "No cookies found");
	        } else {
	            for (int i=0; i<cookies.size(); i++) {
	            	loger.log(Level.DEBUG, "Cookie["+i+"]: '"+cookies.get(i).getName()+":"+cookies.get(i).getValue()+":"+cookies.get(i).getPath()+":"+cookies.get(i).getExpiryDate()+":"+cookies.get(i).getDomain()+"'");
	            }
	        }
		}
	}

	/** Logging de les respostes rebudes pel client http
	 * @param id Identificador de la peticio original
	 * @param httpRequest Request del qual es rep la reposta
	 * @param httpContext Contexte d'execucio. Es necessita per determinar si la resposta es o no de cache.
	 */
	public static void logResponse(String id, HttpRequest httpRequest, HttpContext httpContext) {
		CacheResponseStatus cacheResponseStatus = (CacheResponseStatus) httpContext.getAttribute(CachingHttpClient.CACHE_RESPONSE_STATUS);
		loger.log(Level.DEBUG, "logResponse ["+id+"]["+httpRequest.getRequestLine() + "][cache: " + cacheResponseStatus+"]");
		System.out.println("logResponse ["+id+"]["+httpRequest.getRequestLine() + "][cache: " + cacheResponseStatus+"]");
	}

	/** Logging de les peticions HTTP a l'arxiu de log http
	 * @param s Cadena a incloure en el log
	 */
	public static void log(String s){
		loger.log(Level.DEBUG, s);
	}

}