package pfc.util;

import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/** pfc.util.Er
 * 
 * Utilitats generals relacionades amb expresions regulars
 * 
 */
public class Er {

	private final static Logger loger = Logger.getLogger("er");

	/** Cerca una expressio regular a un text
	 * @param er Expressio regular
	 * @param txt Cadena en la que es fa la cerca
	 * @return Cadena de text trobada
	 * @throws java.lang.IllegalStateException Si no es troba la cadena
	 */
	public static String getValueWithNL(String er, String txt) throws java.lang.IllegalStateException{
		Pattern p = Pattern.compile(er, Pattern.DOTALL);
		Matcher m = p.matcher(txt);
		if (m.find()) {
			for (int i=1;i<=m.groupCount();i++){
				if (m.group(i)!=null) return m.group(i);
			}
			return null;
		} else {
			loger.log(Level.ERROR, "getValueWithNL(er="+er+",txt) NOT FOUND");
			throw new IllegalStateException();
		}
	}

	/** Cerca una expressio regular a un text
	 * @param er Expressio regular
	 * @param txt Cadena en la que es fa la cerca
	 * @return Cadena de text trobada
	 */
	public static String getOptionalValueWithNL(String er, String txt) {
		Pattern p = Pattern.compile(er, Pattern.DOTALL);
		Matcher m = p.matcher(txt);
		if (m.find()) {
			for (int i=1;i<=m.groupCount();i++){
				if (m.group(i)!=null) return m.group(i);
			}
		}
		return null;
	}

	/** Cerca una expressio regular a un text
	 * @param er Expressio regular
	 * @param txt
	 * @return
	 * @throws java.lang.IllegalStateException
	 */
	public static String getValue(String er, String txt) throws java.lang.IllegalStateException{
		Pattern p = Pattern.compile(er);
		Matcher m = p.matcher(txt);
		if (m.find()) {
			for (int i=1;i<=m.groupCount();i++){
				if (m.group(i)!=null) return m.group(i);
			}
			return null;
		} else {
			loger.log(Level.ERROR, "getValue(er="+er+",txt) NOT FOUND");
			throw new IllegalStateException();
		}
	}

	/** Cerca una expressio regular a un text
	 * @param er Expressio regular
	 * @param txt Cadena en la que es fa la cerca
	 * @return Cadena de text trobada
	 */
	public static String getOptionalValue(String er,String txt) {
		Pattern p = Pattern.compile(er);
		Matcher m = p.matcher(txt);
		if (m.find()) {
			for (int i=1;i<=m.groupCount();i++){
				if (m.group(i)!=null) return m.group(i);
			}
		}
		return null;
	}

	/** Cerca a txt els fragments de text que coincideixen amb l'expressio indicada i retorna
	 * a un vector totes les ocurrencies trobades.
	 * @param er Expressio regular
	 * @param txt Text en el que es fa cerca
	 * @param group Nombre del grup de captura que es filtra
	 * @return Vector amb els diferents fragments que coincideixen amb l'expressio regular
	 */
	public static Vector<String> getValues(String er, String txt, int group) {
		Vector<String> vResult = new Vector<String>();
		Pattern p = Pattern.compile(er, Pattern.DOTALL);
		Matcher m = p.matcher(txt);
		while (m.find()) {
			vResult.add(m.group(group));
		}
		return vResult;
	}

	/**
	 * @return Vector amb les diferents ocurrencies de l'expressio regular a la cadena
	 */
	public static Vector<String> getValues(String er, String txt) {
		return getValues(er,txt,0);
	}

	/** Cerca coincidencies d'una expressio regular a una cadena. 
	 * L'expressio regular te grups de captura que es tracten cadascun com una coincidencia diferent
	 * @param er Expressio regular
	 * @param txt Text en el que es fa la cerca
	 * @return Vector amb les coincidencies trobades (a cada posicio les diferents ocurrencies dels diferents grups)
	 */
	public static Vector<Vector<String>> getGroupValues(String er, String txt) {
		Vector<Vector<String>> vResult = new Vector<Vector<String>>();
		Pattern p = Pattern.compile(er, Pattern.DOTALL);
		Matcher m = p.matcher(txt);
		while (m.find()) {
			Vector<String> row = new Vector<String>();
			for (int i=1; i<=m.groupCount(); i++) {
				row.add(m.group(i));
			}
			vResult.add(row);
		}
		return vResult;
	}

	/** Cerca coincidencies a una cadena de text i retorna totes les ocurrencies.
	 * Amb cada ocurrencia la segent cerca es retrocedeix unes posicions per tal de fer coincidir
	 * certs caracters de la cadena amb la segent cerca
	 * @param er Expressio regular
	 * @param txt Text en el que es fa la cerca
	 * @param iBack Nombre de posicions que es retrocedeix el punter dins el match
	 * @return Vector amb les coincidencies trobades
	 */
	public static Vector<String> getValuesAndBack(String er, String txt, int iBack) {
		Vector<String> vResult = new Vector<String>();
		Pattern p = Pattern.compile(er, Pattern.DOTALL);
		Matcher m = p.matcher(txt);
		int next = 0;
		while (m.find(next)) {
			vResult.add(m.group());
			next = m.end() - iBack;
		}
		return vResult;
	}

	/** Reemplaa a una cadena els fragments que coincideixen amb una expressio regular per un nou valor
	 * @param er Expressio regular
	 * @param txt Cadena a tractar
	 * @param newValue Nou valor que reemplaa l'antic
	 * @return Nova cadena
	 */
	public static String replaceValue(String er, String txt, String newValue) {
		return Pattern.compile(er, Pattern.DOTALL).matcher(txt).replaceAll(newValue);
	}

	/** Trim de la cadena sustituint els separadors intermedis i seguits per un unic espai en blanc 
	 * @param txt Cadena a tractar
	 * @return Nova cadena
	 */
	public static String trim(String txt) {
		if (txt!=null) return txt.trim().replaceAll("\\s+"," ");
		else 		   return txt;
	}

	/** Reemplaa els valors que coincideixen per l'expressio regular pel nou valor i aplica un trim a la cadena
	 * @param er Expressio que coincideix amb el text substituit
	 * @param txt Cadena a tractar
	 * @param newValue Nova cadena
	 * @return
	 */
	public static String replaceValueAndTrim(String er, String txt, String newValue){
		if (txt!=null) return trim(replaceValue(er,txt,newValue));
		else           return txt;
	}

	/** Esborra els tags HTML (<...>) i fa un trim de la cadena
	 * @param txt Cadena a tractar
	 * @return Cadena sense tags ni espais al inici/final
	 */
	public static String replaceTagsAndTrim(String txt){
		if (txt!=null) return trim(replaceValue("<.*?>", txt, ""));
		else           return txt;
	}

	/** Esborra els tags html amb el seu contingut i fa un trim de la cadena
	 * @param txt Cadena a tractar
	 * @return Cadena sense tags ni elements entre tags
	 */
	public static String replaceContentTagsAndTrim(String txt){
		if (txt!=null) return trim(replaceValue("<.*?>.*?</.*?>", txt, ""));
		else           return txt;
	}
}