package pfc.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.OutputStreamWriter;
import java.util.Calendar;
import java.util.Vector;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import pfc.conf.Constants;

/** pfc.util.FileUtil
 * 
 * Utilitats d'escriptura/lectura en fitxers
 * Metodes utilitzats per a fer debug de l'informacio
 * El metode mes important es el que escriu l'XML
 * 
 */
public class FileUtil {

	private final static Logger loger = Logger.getLogger("util");

	//Path al servidor del debug dels HTMLs (variable estatic inicialitzada amb el cercador)
	private static String PATH_DEBUG  = null;
	//Path al servidor dels fitxers XML (variable estatic inicialitzada amb el cercador)
	private static String PATH_XML    = null;

	public static void setPathXML(String pathXML){
		PATH_XML = pathXML;
		System.out.println("SET PATH_XML: '" + PATH_XML + "'");
	}

	public static void setPathDebug(String pathDebug){
		PATH_DEBUG = pathDebug;
		System.out.println("SET PATH_DEBUG: '" + PATH_DEBUG + "'");
	}

	/** Lectura del contingut d'un fitxer
	 * @param fitxer Path del fixer que es llegueix
	 * @return Contingut del fitxer
	 */
	public static String readDebugHTML(String fitxer) {
		try {
			long inici = Calendar.getInstance().getTimeInMillis();

			BufferedReader br = new BufferedReader(new FileReader(fitxer));
			StringBuffer sb = new StringBuffer();
			
			//Lectura en blocs del contingut del fitxer
			char[] b = new char[8192];
			int n;
			while ((n = br.read(b)) > 0) {
				sb.append(b, 0, n);
			}

			long fi = Calendar.getInstance().getTimeInMillis();
			loger.log(Level.DEBUG,"readDebugHTML(fitxer="+fitxer+"). time: '" + (fi-inici) + "' ms");
			return sb.toString();
		}
		catch (Exception e) {
			loger.log(Level.ERROR,"readDebugHTML(fitxer="+fitxer+") Exception: "+e);
			return null;
		}
	}

	/** Logging d'una resposta HTTP a un arxiu HTML
	 * @param html Cadena a incloure al fitxer
	 * @param sDir Directori del fitxer
	 * @param sNom Nom de l'arxiu generat
	 */
	public static void logResponseHTML(String text, String sDir, String sFileName) {
		if (pfc.http.HttpLog.isDebugHtmlHTTPActivat()) {
			try {
				write(text,PATH_DEBUG+Constants.FILE_SEP+"http"+Constants.FILE_SEP+sDir+Constants.FILE_SEP,sFileName);
			}
			catch (Exception e) {
				loger.log(Level.ERROR,"logResponseHTML(text,sDir="+sDir+",sFileName="+sFileName+") Exception: " + e);
			}
		}
	}
	
	/** Imprimex el resultat HTML de la cerca a un fitxer
	 * @param text Text a escriure
	 * @param sFileName Nom de l'arxiu generat
	 */
	public static void printHTML(String text, String sFileName) {
		if (pfc.http.HttpLog.isDebugHtmlItemsActivat()) {
			try {
				write(text,
					PATH_DEBUG+Constants.FILE_SEP+"items"+Constants.FILE_SEP+pfc.conf.Constants.sDf.format(Calendar.getInstance().getTime())+Constants.FILE_SEP,
					sFileName);
			} catch (Exception e) {
				loger.log(Level.ERROR,"printHTML(text,sFileName="+sFileName+") Exception: " + e);
			}
		}
	}
	
	/** Imprimeix l'HTML d'un item d'un operador de vols
	 * @param text Text a escriure
	 * @param sFileName Nom de l'arxiu generat
	 */
	public static void printItemFlight(String text, String sFileName) {
		if (pfc.http.HttpLog.isDebugHtmlItemsActivat()) {
			try {
				write(text,
					PATH_DEBUG+Constants.FILE_SEP+"items"+Constants.FILE_SEP+pfc.conf.Constants.sDf.format(Calendar.getInstance().getTime())+Constants.FILE_SEP+"flight"+Constants.FILE_SEP,
					sFileName);
			} catch (Exception e) {
				loger.log(Level.ERROR,"printItemFlight(text,sFileName="+sFileName+") Exception: " + e);
			}
		}
	}

	/** Imprimeix l'HTML d'un item d'un operador d'hotels
	 * @param text Text a escriure
	 * @param sFileName Nom de l'arxiu generat
	 */
	public static void printItemHotel(String text, String sFileName) {
		if (pfc.http.HttpLog.isDebugHtmlItemsActivat()) {
			try {
				write(text,
					PATH_DEBUG+Constants.FILE_SEP+"items"+Constants.FILE_SEP+pfc.conf.Constants.sDf.format(Calendar.getInstance().getTime())+Constants.FILE_SEP+"hotel"+Constants.FILE_SEP,
					sFileName);
			} catch (Exception e) {
				loger.log(Level.ERROR,"printItemHotel(text,sFileName="+sFileName+") Exception: " + e);
			}
		}
	}

	/** Genera un arxiu XML amb l'XML generat
	 * @param text Contingut de l'XML
	 * @param sFileName Nom de l'arxiu generat
	 * @throws Exception
	 */
	public static void printXML(String text, String sFileName) throws Exception {
		write(text, 
			PATH_XML+Constants.FILE_SEP+pfc.conf.Constants.sDf.format(Calendar.getInstance().getTime())+Constants.FILE_SEP,
			sFileName);
	}

	/** Escriu a un arxiu tots els objectes del vector passat per parametre
	 * @param vItems Vector amb els objectes que s'escriuren)
	 * @param sFileName Nom de l'arxiu generat
	 */
	public static void printHTMLItems(Vector<?> vItems, String sFileName) {
		if (pfc.http.HttpLog.isDebugHtmlItemsActivat()) {
			try {
				int i=0;
				String text = "";
				for (Object o: vItems) {
					text += "\n<h1 style=\"margin:0px;padding-left:5px;background-color:#EEEEEE\">"+(++i)+":</h1>\n";
					if (o instanceof pfc.bean.CombinacioBean) {
						text += ((pfc.bean.CombinacioBean)o).toDebugHTML();
					} else {
						text += ((pfc.bean.ItemBean)o).toDebugHTML();
					}
				}
				write(text, 
					PATH_DEBUG+Constants.FILE_SEP+"items"+Constants.FILE_SEP+pfc.conf.Constants.sDf.format(Calendar.getInstance().getTime())+Constants.FILE_SEP,
					sFileName);
			}
			catch (Exception e) {
				loger.log(Level.ERROR,"printHTMLItems(vItems,sFileName="+sFileName+") Exception: " + e);
			}
		}
	}

	/** Escriu un text a un fitxer. 
	 * El directori del fitxer es crea si no existeix (es crea tota l'estructura de directoris)
	 * @param txt Contingut del fitxer
	 * @param sDir Directori del fitxer
	 * @param sFileName Nom del fitxer
	 * @throws Exception
	 */
	private static void write(String txt, String sDir, String sFileName) throws Exception {
		loger.log(Level.DEBUG, "write(txt,sDir="+sDir+",sFileName="+sFileName+")");
		File dir = new File(sDir);
		if (!dir.exists()) dir.mkdirs();
		FileOutputStream fileOutputStream = new FileOutputStream(sDir+sFileName);
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(fileOutputStream));
		bw.write(txt);
		bw.flush();
		bw.close();
		fileOutputStream.flush();
		fileOutputStream.close();
	}

}