package pfc.wrapper;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Vector;

import org.apache.http.HttpEntity;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.client.CookieStore;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.cache.CachingHttpClient;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.ExecutionContext;
import org.apache.http.protocol.HttpContext;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import pfc.BeanPeticioConsulta;
import pfc.bean.ItemBean;
import pfc.exception.PeticioWrapperException;

/** pfc.wrapper.Wrapper
 * 
 * Un Wrapper representa un operador
 * Conte la logica de cerca a un determinat operador i el posterior filtre i captura de dades 
 * Es una classe abstracta (les instancies es creen per a cada operador concret)
 */
public abstract class Wrapper extends Thread {


	protected final static Logger loger = Logger.getLogger("wrapper");
	protected String CLASS_NAME = this.getClass().getName(); //Nom complet de la classe

	//Peticio que es fa a l'operador
	protected BeanPeticioConsulta beanPeticioConsulta = null;

	protected CachingHttpClient httpClient = null;
	protected HttpContext httpContext = null;
	protected ResponseHandler<String> responseHandler = null;
	protected CookieStore cookieStore = null;

	//Una iteracio es una peticio HTTP. 
	//S'utilitza a l'hora de generar els noms dels arxius (sequencials) amb l'html de les respostes.
	protected int iteracio;

	//Resultat (txt) de l'scraping
	protected String resultatScraping = null;
	//Resultat processat de l'scraping una vegada s'han aplicat les expressions regulars
	//Es un vector amb els items trobats
	protected Vector<ItemBean> resultatItemsScraping = null;
	//Text dels fitxers de debug de l'scraping (pren valor si el debug de l'scraping esta activat)
	protected String htmlDebugScraping = null;

	/** Valida les dades de la peticio en relacio a l'operador (wrapper) instanciat
	 * @throws PeticioWrapperException
	 */
	protected abstract void assertPeticioWrapper() throws PeticioWrapperException;
	/** Dona valor a l'informacio extreta amb l'scraping per HTTP (variable resultatScraping)
	 * @throws PeticioWrapperException
	 * @throws IOException
	 */
	protected abstract void getScrapedData()       throws PeticioWrapperException,IOException;
	/**
	 * Analitza l'informaci de l'scraping (resultatScraping) i obte els items trobats (variable resultatItemsScraping)
	 */
	protected abstract void getScrapedItems();

	/**
	 * Constructor d'un wrapper
	 */
	public Wrapper() {
		iteracio = 0;
		httpContext = new BasicHttpContext();
		responseHandler = new ResponseHandler<String>(){
			//Gestor de respostes. Les respostes es retornen dins una variable String.
			public String handleResponse (HttpResponse response) throws IOException{
				HttpEntity entity = response.getEntity();
				if (entity!=null){
					loger.log(Level.DEBUG, "HttEntity for request '" + beanPeticioConsulta.getId() + "' is NOT null");
					String resposta = EntityUtils.toString(entity);

					//Log de l'informacio recuperada
					pfc.util.FileUtil.logResponseHTML(resposta, 
						pfc.conf.Constants.sDf.format(Calendar.getInstance().getTime())+pfc.conf.Constants.FILE_SEP+beanPeticioConsulta.getId(), 
						CLASS_NAME + "#" + (iteracio++) + ".html");
					pfc.http.HttpLog.logResponse(String.valueOf(beanPeticioConsulta.getId()),
						(HttpRequest) httpContext.getAttribute(ExecutionContext.HTTP_REQUEST),
						httpContext);
					pfc.http.HttpLog.printCookies(cookieStore);

					return resposta;
				}
				else {
					loger.log(Level.ERROR, "HttEntity for request id '" + beanPeticioConsulta.getId() + "' is null");
					return null;
				}
			}
		};
	}

	/* (non-Javadoc)
	 * @see java.lang.Thread#run()
	 */
	public void run(){
		long tInici = Calendar.getInstance().getTimeInMillis();
		try {
			loger.log(Level.DEBUG,"[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " INIT");
			System.out.println("[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " INIT");

			//Comprobacio dels parametres enviats
			assertPeticioWrapper();

			//Lectura de l'informacio dels operadors externs
			this.cookieStore = new BasicCookieStore();
			this.httpContext.setAttribute(ClientContext.COOKIE_STORE, cookieStore);

			this.resultatScraping = htmlDebugScraping;
			System.out.println("[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " hay debug: '" + (this.resultatScraping!=null) + "'");
			if (this.resultatScraping == null) {
				long tInici_scraping = Calendar.getInstance().getTimeInMillis();
				getScrapedData();
				long tfi_scraping = Calendar.getInstance().getTimeInMillis();
				loger.log(Level.DEBUG,"["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" end data http scraping '" + (tfi_scraping-tInici_scraping) + "' ms");
			}

			if (this.resultatScraping != null){
				loger.log(Level.DEBUG, "[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " HTML DATA WAS FOUND");
				System.out.println("[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " HTML DATA WAS FOUND");
				long tInici_scraping = Calendar.getInstance().getTimeInMillis();
				getScrapedItems();
				long tfi_scraping = Calendar.getInstance().getTimeInMillis();
				loger.log(Level.DEBUG,"["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" end data ER process '" + (tfi_scraping-tInici_scraping) + "' ms");
			}
		}
		catch (PeticioWrapperException pwe){
			//El format de la peticio no es suportat pel wrapper
			loger.log(Level.ERROR, "[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " PeticioWrapperException: '" + pwe.getMessage() + "'");
			pwe.printStackTrace();
			System.out.println("[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " ERROR1");
		}
		catch (java.lang.IllegalStateException ise){
			//Excepcio davant expresions regulars no coincidents 
			//(Per exemple, si no existeixen resultats o l'html original te canvis no contemplats)
			
			//S'escriu la traa de l'error al fitxer de log
			Writer writer = new StringWriter();
			PrintWriter printWriter = new PrintWriter(writer);
			printWriter.print("No matching criteria was found");
		    printWriter.println();
		    ise.printStackTrace(printWriter);
			loger.log(Level.ERROR, "[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " IllegalStateException: '"+writer.toString()+"'");
			ise.printStackTrace();
			System.out.println("[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " ERROR2");
		}
		catch (IOException ioe){
			loger.log(Level.FATAL, "[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " IOException: '"+ioe + "'");
			ioe.printStackTrace();
			System.out.println("[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " ERROR3");
		}
		catch (Exception e){
			loger.log(Level.FATAL, "[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " Fatal error: '" + e + "'");
			e.printStackTrace();
			System.out.println("[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " ERROR4");
		}
		long tFi = Calendar.getInstance().getTimeInMillis();
		loger.log(Level.DEBUG, "[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " END (" + (tFi - tInici) + "ms)");
		System.out.println("[" + this.beanPeticioConsulta.getId() + "] " + CLASS_NAME + " END (" + (tFi - tInici) + "ms)");
	}

	public Vector<ItemBean> getResultatScraping() {
		if (resultatItemsScraping!=null) return resultatItemsScraping;
		else 							 return new Vector<ItemBean>();
	}

	public void setHtmlDebugScraping(String htmlDebugScrapingParam) {
		htmlDebugScraping = (htmlDebugScrapingParam != null) ? htmlDebugScrapingParam : "";
	}

	public void setHttpClient(CachingHttpClient httpClient) {
		this.httpClient = httpClient;
	}

	/** Suspensio del proces (sleep)
	 * @param milisegons Nombre de milisegons
	 */
	public void dormir(int milisegons) {
		try   { sleep(milisegons); }
		catch (InterruptedException e){ e.printStackTrace(); }
	}

	public void setBeanPeticioConsulta(BeanPeticioConsulta beanPeticioConsulta){
		this.beanPeticioConsulta = beanPeticioConsulta;
	}

	/////////////////////////////////////////////
	//Llistat de parametres enviats dins les cadenes de les peticios HTTP pels wrappers instanciats
	/////////////////////////////////////////////
	public String getDataInici(String format){
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(format);
		return simpleDateFormat.format(this.beanPeticioConsulta.getcDesde().getTime());
	}
	public String getDataFi(String format){
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(format);
		return simpleDateFormat.format(this.beanPeticioConsulta.getcHasta().getTime());
	}
	public String getDiaInici(){
		return String.valueOf(this.beanPeticioConsulta.getcDesde().get(Calendar.DAY_OF_MONTH));
	}
	public String getMesInici(){
		return String.valueOf(this.beanPeticioConsulta.getcDesde().get(Calendar.MONTH)+1);
	}
	public String getDiaFi(){
		return String.valueOf(this.beanPeticioConsulta.getcHasta().get(Calendar.DAY_OF_MONTH));
	}
	public String getMesFi(){
		return String.valueOf(this.beanPeticioConsulta.getcHasta().get(Calendar.MONTH)+1);
	}

}