package pfc.wrapper.flight.flights;
import java.io.IOException;
import java.util.Calendar;
import java.util.Vector;

import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.log4j.Level;

import pfc.bean.ItemBean;
import pfc.bean.flight.VuelingBean;
import pfc.conf.Ciutats;
import pfc.util.Er;
import pfc.exception.PeticioWrapperException;
import pfc.http.HttpLog;

/** pfc.wrapper.flight.flights.Vueling
 * 
 * Wrapper de l'operador Vueling
 */
public class Vueling extends pfc.wrapper.flight.WrapperFlight {

	protected void assertPeticioWrapper() throws PeticioWrapperException {
		if (this.beanPeticioConsulta.getNumTotalAdults(12)>9) throw new PeticioWrapperException(PeticioWrapperException.MAX_TOTAL_ADULTS);
		if (this.beanPeticioConsulta.getNumTotalNins(12)>9) throw new PeticioWrapperException(PeticioWrapperException.MAX_TOTAL_NINS);
		if (this.beanPeticioConsulta.getNumTotalBebes(1)>9) throw new PeticioWrapperException(PeticioWrapperException.MAX_TOTAL_BEBES);
		//No es permeten reserves amb data de tornada major a sis meses
		Calendar avui = Calendar.getInstance();
		avui.add(Calendar.MONDAY, 6);
		if (this.beanPeticioConsulta.getcHasta().after(avui)){
			throw new PeticioWrapperException(PeticioWrapperException.RANG_DATES);
		}
	}

	protected String getCiutat(String ciutat) throws PeticioWrapperException{
		if (ciutat.equals(Ciutats.CIUTAT_SEVILLA))		return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_MALAGA))		return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_PALMA)) 		return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_IBIZA)) 		return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_MENORCA))		return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_LASPALMAS))	return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_TENERIFE)) 	return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_BARCELONA)) 	return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_VALENCIA))		return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_ALICANTE)) 	return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_CORUNYA)) 		return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_VIGO)) 		return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_MADRID)) 		return this.getParamAeroport(ciutat);
		if (ciutat.equals(Ciutats.CIUTAT_BILBAO)) 		return this.getParamAeroport(ciutat);
		throw new PeticioWrapperException(PeticioWrapperException.CIUTAT_NO_CONFIGURADA.replaceAll("\\{0\\}", ciutat));
	}

	protected void getScrapedData() throws PeticioWrapperException,IOException
	{
		//INICIALITZACIONS
		String resposta = null;
		String uri = null;

		/* ********************************************** */
		//PETICIO 1
		uri = "http://www.vueling.com";

		HttpLog.log("["+beanPeticioConsulta.getId() + "][HttpGet]:" + uri);
		HttpGet peticio_1 = new HttpGet(uri);
    	resposta = httpClient.execute(peticio_1,responseHandler,httpContext);

		/* ********************************************** */
		//PETICIO 2
		uri = "http://www.vueling.com/booking/booking/selecciona-tu-vuelo?";
		uri += "event=search";
		uri += "&module=SB";
		uri += "&page=SEARCH";
		uri += "&language=ES";
		uri += "&mode=";
		uri += "&sid=";	
		uri += "&ref=";
		uri += "&travel=2";
		uri += "&from1=" + pfc.conf.Util.encode(getCiutat(this.beanPeticioConsulta.getOrigen()));
		uri += "&to1=" + pfc.conf.Util.encode(getCiutat(this.beanPeticioConsulta.getDesti()));
		uri += "&from2=" + pfc.conf.Util.encode(getCiutat(this.beanPeticioConsulta.getDesti()));
		uri += "&to2=" + pfc.conf.Util.encode(getCiutat(this.beanPeticioConsulta.getOrigen()));	
		uri += "&departDay1=" + this.getDiaInici();
		uri += "&departMonth1=" + this.getDataInici("yyyyMM");
		uri += "&displayDate1=" + this.getDataInici("EEEE+dd+MMMM,+yyyy");
		uri += "&depart1FlexBy=0000";
		uri += "&departDay2=" + this.getDiaFi();
		uri += "&departMonth2=" + this.getDataFi("yyyyMM");
		uri += "&displayDate2=" + this.getDataFi("EEEE+dd+MMMM,+yyyy");
		uri += "&depart2FlexBy=0000";
		uri += "&fechas=0000";
		uri += "&ADULT=" + this.beanPeticioConsulta.getNumTotalAdults(12);
		uri += "&defaultADULT=-1";
		uri += "&CHILD=" + this.beanPeticioConsulta.getNumTotalPax(2,11);
		uri += "&defaultCHILD=-1";
		uri += "&INFANT=" + this.beanPeticioConsulta.getNumTotalBebes(1);
		uri += "&defaultINFANT=-1";
		uri += "&toCity1=Destino";
		uri += "&toCity2=???";
		uri += "&departDate1=" + this.getDataInici("yyyyMMdd");
		uri += "&departDate2=" + this.getDataFi("yyyyMMdd");
		uri += "&numberMarkets=2";
		uri += "&cualquier=";
		uri += "&nom_cualquier=";
		uri += "&m1_cualquier=";
		uri += "&m2_cualquier=";
		uri += "&frdisc=";
		uri += "&mode_orig=";
		uri += "&zanpid=";
		uri += "&utm_campaign=";
		uri += "&tduid=";
		String modeTESTAB = Er.getOptionalValue("mode_TESTAB';elem.value = '([^']*)'",resposta);
		if (modeTESTAB!=null) {
			uri += "&mode_TESTAB="       + modeTESTAB;
			uri += "&mode_TESTABClassB=" + modeTESTAB;
		}

		dormir(1);
		HttpLog.log("["+beanPeticioConsulta.getId() + "][HttpPost]:" + uri);
		HttpPost peticio_2 = new HttpPost(uri);
    	resposta = httpClient.execute(peticio_2,responseHandler,httpContext);

		resultatScraping = resposta;
	}
	
	protected void getScrapedItems(){
		Vector<ItemBean> vVolBean = new Vector<ItemBean>();

		Vector<Vector<String>> items = Er.getGroupValues(">Ida(?:</span|</div)(.*?)Vuelta(?:</span|</div)(.*)",resultatScraping);
		loger.log(Level.DEBUG,"["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" main content found");
		System.out.println("["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" main content found");

		if (items.size()>0) {
			//Dates anades
			int indexItem=0;

			Vector<String> vAnades = Er.getValues("class=\"destacado.*?\".*?(<table.*?</table>.*?)*</tr",items.get(0).get(0));
			loger.log(Level.DEBUG,"["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" " + vAnades.size() + " anadas items found");
			System.out.println("["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" " + vAnades.size() + " anadas items found");
			
			Vector<Object[]> vItemsAnades = new Vector<Object[]>();
			for (String anada: vAnades) {
				loger.log(Level.DEBUG,"["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" item anada["+(++indexItem)+"/"+vAnades.size()+"]");
				pfc.util.FileUtil.printItemFlight(anada, CLASS_NAME+"#anada#item_" + indexItem + ".html");
				Vector<Vector<String>> vCols = Er.getGroupValues("<td.*?>(.*?(<table.*?>.*?</table>.*?)*)</td>",anada);
				Vector<String> preus = Er.getValues("(\\d+(\\.\\d+)?)(<![^>]*>)?\\s*&euro;",vCols.get(1).get(0),1);
				String dataInici = Er.replaceTagsAndTrim(vCols.get(2).get(0).replaceAll("<br.*?>", " "));

				//Rutes
				Vector<String[]> vRutes = new Vector<String[]>();
				Vector<Vector<String>> vColsHores = Er.getGroupValues("<td.*?class=\"hora\">(.*?)</td",anada);
				if (vColsHores.size()>0) {
					Vector<Vector<String>> vColsRutes = Er.getGroupValues("<td.*?class=\"ruta\">(.*?)</td",anada);
					for (int indexHora=0; indexHora<vColsHores.size(); indexHora++) {
						String hora = vColsHores.get(indexHora).get(0).replaceAll("<br.*?>", " ");
						hora = Er.replaceTagsAndTrim(hora);
						String ruta = vColsRutes.get(indexHora).get(0).replaceAll("<br.*?>", "/");
						ruta = Er.replaceTagsAndTrim(ruta);
						String[] array = {hora,ruta};
						vRutes.add(array);
					}
				}
				else {
					String vColHora = vCols.get(vCols.size()-2).get(0);
					String hora = vColHora.replaceAll("<br.*?>", " ");
					hora = Er.replaceTagsAndTrim(hora);
					String vColRuta = vCols.get(vCols.size()-1).get(0);
					String ruta = vColRuta.replaceAll("<br.*?>", " ");
					ruta = Er.replaceTagsAndTrim(ruta);

					String[] array = {hora,ruta};
					vRutes.add(array);
				}
				vItemsAnades.add(new Object[] {
					preus.get(0),
					preus.size()>1?preus.get(1):null,
					dataInici,
					vRutes});

			}
			//Dates tornades
			indexItem=0;

			Vector<String> vTornades = Er.getValues("class=\"destacado.*?\".*?(<table.*?</table>.*?)*</tr",items.get(0).get(1));
			loger.log(Level.DEBUG,"["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" " + vTornades.size() + " tornades items found");
			System.out.println("["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" " + vTornades.size() + " tornades items found");

			Vector<Object[]> vItemsTornades = new Vector<Object[]>();
			for (String tornada: vTornades) {
				loger.log(Level.DEBUG,"["+this.beanPeticioConsulta.getId()+"] "+CLASS_NAME+" item tornada["+(++indexItem)+"/"+vTornades.size()+"]");
				pfc.util.FileUtil.printItemFlight(tornada, CLASS_NAME+"#tornada#item_" + indexItem + ".html");
				Vector<Vector<String>> vCols = Er.getGroupValues("<td.*?>(.*?(<table.*?>.*?</table>.*?)*)</td>",tornada);
				Vector<String> preus = Er.getValues("(\\d+(\\.\\d+)?)(<![^>]*>)?\\s*&euro;",vCols.get(1).get(0),1);
				String dataInici = Er.replaceTagsAndTrim(vCols.get(2).get(0).replaceAll("<br.*?>", " "));

				//Rutes
				Vector<Vector<String>> vColsHores = Er.getGroupValues("<td.*?class=\"hora\">(.*?)</td",tornada);
				Vector<String[]> vRutes = new Vector<String[]>();
				if (vColsHores.size()>0) {
					Vector<Vector<String>> vColsRutes = Er.getGroupValues("<td.*?class=\"ruta\">(.*?)</td",tornada);
					for (int indexHora=0; indexHora<vColsHores.size(); indexHora++) {
						String hora = vColsHores.get(indexHora).get(0).replaceAll("<br.*?>", " ");
						hora = Er.replaceTagsAndTrim(hora);
						String ruta = vColsRutes.get(indexHora).get(0).replaceAll("<br.*?>", "/");
						ruta = Er.replaceTagsAndTrim(ruta);
						String[] array = {hora,ruta};
						vRutes.add(array);
					}
				}
				else {
					String vColHora = vCols.get(vCols.size()-2).get(0);
					String hora = vColHora.replaceAll("<br.*?>", " ");
					hora = Er.replaceTagsAndTrim(hora);
					String vColRuta = vCols.get(vCols.size()-1).get(0);
					String ruta = vColRuta.replaceAll("<br.*?>", " ");
					ruta = Er.replaceTagsAndTrim(ruta);

					String[] array = {hora,ruta};
					vRutes.add(array);
				}

				vItemsTornades.add(new Object[]{
					preus.get(0),
					preus.size()>1?preus.get(1):null,
					dataInici,
					vRutes});
			}
			//Combinacions
			for (Object[] anada: vItemsAnades) {
				for (Object[] tornada: vItemsTornades) {
					VuelingBean vuelingBean = new VuelingBean(beanPeticioConsulta);
					vuelingBean.setAnada(anada);
					vuelingBean.setTornada(tornada);
					vVolBean.add(vuelingBean);
				}			
			}
		}
		resultatItemsScraping = new Vector<pfc.bean.ItemBean>(vVolBean);
	}
}